/*
 * DMIOST.C - DM I/O symbol table module.
*/

#include "env.h"  /* must appear before first C statement */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <malloc.h>

#include "hash.h"
#include "dmio.h"

static void sym_error( char *, ... );

/* INSTALL                                                                 */
/* Add resource info to symbol table.  Note that 0th symtab element        */
/* is not used.                                                            */

stEntry *install(symtab s, char *name, cobolType type, ulong offset,
	ulong length)
{
	stEntry *pstEntry;

	if(NULL == (pstEntry = findsym((HASH) s, name)))
		if(NULL == (pstEntry = hadd((HASH) s, name, sizeof(stEntry))))
	      sym_error("Too many symbols for symbol table at %s", name);

	pstEntry->type = type;
	pstEntry->offset = offset;
	pstEntry->length = length;

	return(pstEntry);
}


/* FINDSYM                                                                 */
/* Attempts to locate symbol in symbol table.  Returns stEntry *           */
/* where symbol found, or NULL if symbol not found.                        */

stEntry *findsym(symtab s, char *name)
{
	return(hfind((HASH) s, name));
}


/* dmstConstruct
 * Creates a symbol table which can hold a maximum of nsymbols.
 */

symtab dmstConstruct(size_t nsymbols)
{
	return((symtab) hash_create(127, MAX_COBOL_NAME, NULL, NULL, NULL));
}


/* dmstDestruct
 * Destroys a symbol table.
 */

void dmstDestruct(symtab *s)
{
	hdestroy(s);
}

void dmstIterate(symtab s, void (*pIterFunc)(symtab s, stEntry *pstEntry))
{
	stEntry *pstEntry;

	FOR_EACH_HASH_ENTRY(((HASH) s), pstEntry, stEntry *)
		(*pIterFunc)(s, pstEntry);
	END_FOR_HASH
}

/* SYM_ERROR                                                               */
/* Report symbol table handling errors.                                    */

static void sym_error(char *fmt, ... )
{
   va_list  ap;

   /* can add sym_error prefix message here, if desired */
   va_start(ap, fmt);
   vfprintf(stderr, fmt, ap);
   va_end(ap);

   exit(1);
}
